<?php

try {
    require_once(dirname(__FILE__) . '/application.php');
    require_once(dirname(__FILE__) . '/setup.php');
    require_once(dirname(__FILE__) . '/lib/export.php');
    $tplPath = dirname(__FILE__) . "/tpl/export.html";

    @set_time_limit(0);

    class Surfacer_Export extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {
            $DB = DB::singleton(SetupCommon::dsn());
            $SQL = SQL::newSelect('blog');
            $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

            foreach ( $all as $row ) {
                $vars = array(
                    'bid' => $row['blog_id'],
                    'name' => $row['blog_name'],
                    'code' => $row['blog_code'],
                    'parent' => $row['blog_parent'],
                );
                if ( $row['blog_id'] == $this->Post->get('bid') ) $vars += array('checked' => ' checked="checked"');
                $Tpl->add(array('blog:loop', $root), $vars);
            }

            if ( version_compare(PHP_VERSION, '5.0.0', '>=') ) {
                $Tpl->add(array('php5', $root));
            } else {
                $Tpl->add(array('php4', $root));
            }
        }

        function validate()
        {
            if ( $this->Post->isNull('bid') ) {
                $this->Error['blog_select'] = true;
            } else {
                $DB = DB::singleton(SetupCommon::dsn());
                $SQL = SQL::newSelect('blog');
                $SQL->addWhereOpr('blog_id', $this->Post->get('bid'));
                if ( !$DB->query($SQL->get(SetupCommon::dsn()), 'exec') ) {
                    $this->Error['blog_exist'] = true;
                }
            }

            $path = ROOT_DIR . $this->Post->get('path', SETUP_DIR);
            $_POST['path'] = $this->Post->get('path', SETUP_DIR);
            if ( is_dir($path) ) {
                if ( PHP_OS == 'WIN32' or PHP_OS == 'WINNT' ) {
                    if ( !is_readable($path) or !is_writable($path) ) $this->Error['permission'] = true;
                } else {
                    if ( !is_readable($path) or !is_writable($path) or !is_executable($path) ) $this->Error['permission'] = true;
                }
            } else {
                $this->Error['directory'] = true;
            }

            if ( $this->Post->isNull('name') ) {
                $this->Error['name_input'] = true;
            } else {
                $name = $this->Post->get('name');
                if ( preg_match('@[^\w\d\-\.]@', $name) ) {
                    $this->Error['name_valid'] = true;
                }
            }

            $this->detectStep();
        }

        function business()
        {
            @set_time_limit(0);
            ini_set('memory_limit', '1G');

            $tables = array('category', 'column', 'config', 'comment', 'config_set',
                'dashboard', 'entry', 'field', 'form', 'fulltext',
                'module', 'rule', 'tag', 'trackback', 'schedule', 'layout_grid',
            );

            $bid = $this->Post->get('bid');
            $name = $this->Post->get('name');
            //$path = ROOT_DIR.$this->Post->get('path', 'archives').'/'.$name';
            $sep = $this->Post->get('marker');

            $Export = new Export($bid, $tables, $sep);
            $yaml = $Export->getYaml();

            if ( $this->Post->get('ext') == 'zip' && !class_exists('ZipArchive') ) {

                $this->Error['zip_archive'] = true;

            } elseif ( $this->Post->get('ext') == 'zip' || $this->Post->get('ext') == 'gzip' ) {
                include_once(dirname(__FILE__) . '/lib/compose.php');
                $array = $Export->getArray();

                /**
                 * archives & themes temporary copy for exportable
                 */
                if ( !empty($array) && is_array($array) ) {
                    $tmp = "$name/";
                    $tmp_thm = "$name/themes/";
                    $tmp_arc = "$name/archives/";

                    if ( @mkdir($tmp) && @mkdir($tmp_thm) && @mkdir($tmp_arc) ) {

                        /**
                         * detected Using Theme
                         */
                        foreach ( $array['config'] as $conf ) {
                            if ( $conf['config_key'] == 'theme' ) {

                                $theme_name = $conf['config_value'];
                                $theme_path = ROOT_DIR . 'themes/' . $theme_name . '/';
                                $theme_tmp = $tmp_thm . $theme_name . '/';

                                $from = SetupCommon::fixPathSeparator($theme_path);
                                $to = SetupCommon::fixPathSeparator($theme_tmp);

                                if ( @mkdir($to) && file_exists($from) ) {
                                    SetupCommon::copyDirectories($from, $to);
                                    SetupCommon::copyFiles($from, $to);
                                }
                            }
                        }

                        /**
                         * detected Blog Archives
                         */
                        $archive_path = ROOT_DIR . 'archives/' . sprintf("%03d", $bid) . '/';
                        if ( file_exists($archive_path) ) {
                            $archive_tmp = $tmp_arc . sprintf("%03d", $bid) . '/';

                            $from = SetupCommon::fixPathSeparator($archive_path);
                            $to = SetupCommon::fixPathSeparator($archive_tmp);

                            if ( @mkdir($to) && file_exists($from) ) {
                                SetupCommon::copyDirectories($from, $to);
                                SetupCommon::copyFiles($from, $to);
                            }
                        }

                        /**
                         * make YAML dumpData
                         */
                        $_yaml = fopen($tmp . $name . '.yaml', "w+");
                        flock($_yaml, LOCK_EX);
                        fputs($_yaml, $yaml);
                        flock($_yaml, LOCK_UN);
                        chmod($tmp . $name . '.yaml', 0666);
                    }
                }

                /**
                 * compose zip
                 */
                if ( file_exists($tmp) ) {
                    if ( $this->Post->get('ext') == 'zip' && version_compare(PHP_VERSION, '5.0.0', '>=') && class_exists('ZipArchive') ) {
                        $zip = "$name.zip";
                        $pack = new composeZip();
                        $pack->compress($zip, $tmp);
                        SetupCommon::deleteDirectory($tmp);

                        /**
                         * send blogBinaryData buff
                         */
                        $size = filesize($zip);
                        mb_http_output("pass");
                        header("Content-type: application/zip");
                        header("Content-Disposition: attachment; filename=$name.zip");
                        header("Content-length: " . $size);
                        readfile($zip);
                        @chmod($zip, 0777);
                        @unlink($zip);
                    } elseif ( $this->Post->get('ext') == 'gzip' ) {
                        $gzip = "$name.tar.gz";
                        exec("tar cvf $gzip $tmp");

                        /**
                         * send blogBinaryData buff
                         */
                        $size = filesize($gzip);
                        mb_http_output("pass");
                        header("Content-type: application/x-gzip");
                        header("Content-Disposition: attachment; filename=$name.tar.gz");
                        header("Content-length: " . $size);
                        readfile($gzip);
                        @chmod($gzip, 0777);
                        @unlink($gzip);
                    }
                }

            } elseif ( $this->Post->get('ext') == 'yaml' ) {
                header('Content-Disposition: attachment; filename=' . mb_convert_encoding($name . '.yaml', 'SJIS'));
                header('Content-Length: ' . strlen($yaml));

                if ( strpos(UA, 'MSIE') ) {
                    header('Content-Type: text/download');
                } else {
                    header('Content-Type: application/octet-stream');
                }
                die($yaml);
            } else {
                $this->Error['zip_archive'] = true;
            }

            $this->detectStep();
        }
    }

    new Surfacer_Export($tplPath);

    App::checkException();
} catch ( Exception $e ) {
    App::showError($e);
}
